/**
 * @file wx_image_c_api.h
 * SDK图片操作API
 */

#ifndef WX_IMAGE_C_API_H_
#define WX_IMAGE_C_API_H_

#include <stdint.h>
#include <stddef.h>

#include "wx_api.h"

#ifdef __cplusplus
extern "C" {
#endif

/** WXImage_t 图像数据格式 */
typedef enum WXImageDataFormat_e{
    /** 未知格式 */
    WXImgDF_UNK = 0,

    /** 三通道RGB格式 */
    WXImgDF_RGB = 1,

    /** 四通道RGBA格式 */
    WXImgDF_RGBA,

    /** 单通道ALPHA格式 */
    WXImgDF_ALPHA8,

    /** 单通道浮点值 */
    WXImgDF_FLOAT32
} WXImageDataFormat_e;

/** WXImage_t结构
 * @see wonxai::WXImage */
typedef struct WXImage_t WXImage_t;

/**
 * 从文件创建一个WXImage_t对象
 * @see wonxai::WXImage::fromFile
 * @return
 * <pre>
 *   成功 - 0
 *   失败 - 错误码
 * </pre>
 */
WX_API int WXImage_FromFile(const char *fileName, WXImage_t **img_p);

/**
 * 从图片文件数据创建一个WXImage_t对象
 * @see wonxai::WXImage::fromEncodedData
 * @return
 * <pre>
 *   成功 - 0
 *   失败 - 错误码
 * </pre>
 */
WX_API int WXImage_FromEncodedData(const uint8_t *data, size_t size, WXImage_t **img_p);

/**
 * 创建一副纯色图，颜色值由r，g，b三个值指定
 * @see wonxai::WXImage::fromColor
 * @return
 * <pre>
 *   成功 - 0
 *   失败 - 错误码
 * </pre>
 */
WX_API int WXImage_FromColor(int width, int height,
                             uint8_t r, uint8_t g, uint8_t b,
                             WXImage_t **img_p);

/**
 * 根据给定的图像像素数据，创建WXImage_t对象
 * @see wonxai::WXImage::fromBuffer
 * @return
 * <pre>
 *   成功 - 0
 *   失败 - 错误码
 * </pre>
 */
WX_API int WXImage_FromBuffer(int width, int height,
                              const uint8_t *rgbData,
                              WXImage_t **img_p);

/** 释放一个WXImage_t 对象*/
WX_API void WXImage_Release(WXImage_t *img);

/** 将图片保存到文件，支持jpg和png图片 */
WX_API int WXImage_ToFile(WXImage_t *img, const char *fileName);

/**
 * 将图片保存为编码后的数据，支持jpg和png格式
 * @see wonxai::WXImage::toEncodedData
 * @note encodedData的空间由调用者自行分配和管理
 * @param[in] img  WXImage对象
 * @param[in] imgType  图片编码类型：jpg or png
 * @param[out] encodedData - 用于接收编码后的二进制数据
 * @param[in] encodedDataCapcity - encodedData空间长度
 * @param[out] encodedDataSize - 返回的编码后的图片数据的长度
 * @return
 * <pre>
 *   成功 - 0
 *   失败 - 错误码
 * </pre>
 */
WX_API int WXImage_ToEncodedData(WXImage_t *img,
                                 const char *imgType, /* "jpg" or "png" */
                                 uint8_t *encodedData,
                                 size_t encodedDataCapcity,
                                 size_t *encodedDataSize);

/** 返回图片宽度 */
WX_API int WXImage_Width(WXImage_t *img);

/** 返回图片高度 */
WX_API int WXImage_Height(WXImage_t *img);

/** 返回图片数据，调用者不能从外面释放，也不需要单独释放 */
WX_API uint8_t *WXImage_Data(WXImage_t *img);

/** 返回图片数据格式 */
WX_API WXImageDataFormat_e WXImage_DataFormat(WXImage_t *img);

#ifdef __cplusplus
}
#endif

#endif // WX_IMAGE_C_API_H_
